open Big_int

module type FIELD = sig
  type t 

  val zero : t
  val one : t

  val add : t -> t -> t

  val multiply : t -> t -> t

  val negate : t -> t

  (* return the multiplicative inverse of the argument.
   * Behavior is undefined if the argument is zero.
   *)
  val inverse : t -> t

  val to_string : t -> string

  val num_of_big_int : big_int -> t

  val num_of_int : int -> t
end

module type OrderedField = sig
  include FIELD

  type cmp = Less | Greater | Equal
  val cmp : t -> t -> cmp
end

module Floats : OrderedField with type t = float

module Q : OrderedField 
  with type t = big_int * big_int

module R : OrderedField with type t = big_int -> Q.t