
(** A non-empty splay tree contains a left subtree, a value, and a right tree. *)
type 'a splay_tree = Leaf | Node of 'a splay_tree * 'a * 'a splay_tree


(** [splay t x] returns a splay tree with [x] splayed to the top of [t]. *)
val splay : 'a splay_tree -> 'a -> 'a splay_tree

(** [lookup t x] returns [(Some x, t')] if [x] was found in [t] where [t'] is the
 *  splay tree after a lookup. Returns [(None, t')] if [x] is not in [t]. *)
val lookup : 'a splay_tree -> 'a -> 'a option * 'a splay_tree

(** [insert t x] returns a splay tree with the value [x] inserted into [t].
 *  The returned tree should not have any duplicate values.*)
val insert : 'a splay_tree -> 'a -> 'a splay_tree

(** [delete t x] returns a splay tree with the value [x] removed from [t]. *)
val delete : 'a splay_tree -> 'a -> 'a splay_tree
